/* add-id.c
 *
 * vim:smartindent ts=8:sts=2:sta:et:ai:shiftwidth=2
 ****************************************************************
 * Copyright (C) 2001, 2002, 2003  Tom Lord
 *
 * See the file "COPYING" for further information about
 * the copyright and warranty status of this work.
 */


#include "config-options.h"
#include "po/gettext.h"
#include "hackerlab/cmd/main.h"
#include "libarch/inv-ids.h"
#include "commands/cmd.h"
#include "commands/add-id.h"
#include "commands/version.h"



static t_uchar * usage = N_("[options] file ...");

#define OPTS(OP) \
  OP (opt_help_msg, "h", "help", 0, \
      N_("Display a help message and exit.")) \
  OP (opt_long_help, "H", 0, 0, \
      N_("Display a verbose help message and exit.")) \
  OP (opt_version, "V", "version", 0, \
      N_("Display a release identifier string\n" \
      "and exit.")) \
  OP (opt_id, "i", "id ID", 1, \
      N_("Specify ID, instead of using auto-generated id."))

t_uchar arch_cmd_add_id_help[] = N_("add an explicit inventory id\n"
                                   "Create an explicit inventory id for FILE (which may be a\n"
                                   "regular file, symbolic link, or directory).\n");


enum options
{
  OPTS (OPT_ENUM)
};

static struct opt_desc opts[] =
{
  OPTS (OPT_DESC)
    {-1, 0, 0, 0, 0}
};



int
arch_cmd_add_id (t_uchar *program_name, int argc, char * argv[])
{
  int o;
  struct opt_parsed * option;
  char * id = 0;

  safe_buffer_fd (1, 0, O_WRONLY, 0);

  option = 0;

  while (1)
    {
      o = opt_standard (lim_use_must_malloc, &option, opts, &argc, argv, program_name, usage, libarch_version_string, arch_cmd_add_id_help, opt_help_msg, opt_long_help, opt_version);
      if (o == opt_none)
        break;
      switch (o)
        {
        default:
          safe_printfmt (2, "unhandled option `%s'\n", option->opt_string);
          panic ("internal error parsing arguments");

        usage_error:
          opt_usage (2, argv[0], program_name, usage, 1);
          exit (1);

          /* bogus_arg: */
          safe_printfmt (2, "ill-formed argument for `%s' (`%s')\n", option->opt_string, option->arg_string);
          goto usage_error;

       case opt_id:
           {
                  id = str_save(0, option->arg_string);
                  break;
           }
        }
    }

  if (argc < 2)
    goto usage_error;

  if (id && argc > 2)
    {
      safe_printfmt (2, "--id may only be supplied for the addition of one id. You supplied %d ids.\n", argc - 1);
      goto usage_error;
    }

  {
    int index;

    for (index = 1; index < argc; ++index)
      {
        t_uchar * added_id = arch_add_id(argv[index], id);
        if (added_id == NULL)
          {
            safe_printfmt (2, "Failed to add id for path '%s'\n", argv[index]);
            return 1;
          }
        lim_free (0, added_id);
      }
  }


  return 0;
}



/* tag: Tom Lord Wed May 14 12:24:43 2003 (add-tag.c)
 */
